// 页面加载完成后执行
document.addEventListener('DOMContentLoaded', function() {
    // 获取所有产品图片
    const productImages = document.querySelectorAll('.product-image');
    
    // 为每个图片添加错误处理
    productImages.forEach(img => {
        img.onerror = function() {
            this.src = '/static/products/default.jpg';
        };
    });

    initCharts();
    filterByDate('today'); // 默认显示今日数据

    // 添加刷新按钮事件监听
    document.getElementById('refreshBtn').addEventListener('click', function() {
        refreshData();
        // 重置倒计时
        countdown = 30 * 60;
    });
});

// 设置倒计时刷新
let countdown = 30 * 60; // 30分钟
let currentFilter = 'today'; // 保存当前的筛选条件

function updateCountdown() {
    const minutes = Math.floor(countdown / 60);
    const seconds = countdown % 60;
    const timeString = `${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`;
    
    const countdownElements = document.querySelectorAll('#countdown');
    countdownElements.forEach(element => {
        element.textContent = timeString;
    });
    
    countdown--;
    
    if (countdown < 0) {
        countdown = 30 * 60; // 重置为30分钟
        refreshData(); // 刷新数据
    }
}

// 每秒更新倒计时
setInterval(updateCountdown, 1000);

let productChart, hourlyChart;

function initCharts() {
    const productCtx = document.getElementById('productChart').getContext('2d');
    const hourlyCtx = document.getElementById('hourlyChart').getContext('2d');

    productChart = new Chart(productCtx, {
        type: 'bar',
        data: {
            labels: [],
            datasets: [{
                label: 'Returns by Product',
                data: [],
                backgroundColor: 'rgba(54, 162, 235, 0.5)',
                borderColor: 'rgba(54, 162, 235, 1)',
                borderWidth: 1
            }]
        },
        options: {
            responsive: true,
            maintainAspectRatio: false,
            scales: {
                y: {
                    beginAtZero: true,
                    ticks: {
                        stepSize: 1
                    }
                }
            }
        }
    });

    hourlyChart = new Chart(hourlyCtx, {
        type: 'line',
        data: {
            labels: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
            datasets: [{
                label: 'Returns by Day of Week',
                data: [0, 0, 0, 0, 0, 0, 0],
                fill: false,
                borderColor: 'rgba(75, 192, 192, 1)',
                tension: 0.1
            }]
        },
        options: {
            responsive: true,
            maintainAspectRatio: false,
            scales: {
                y: {
                    beginAtZero: true,
                    ticks: {
                        stepSize: 1
                    }
                }
            }
        }
    });
}

// 更新统计数据
function updateStats(data) {
    document.getElementById('totalReturns').textContent = data.total_returns;
    document.getElementById('totalPoints').textContent = data.total_points;
    document.getElementById('activeUsers').textContent = data.active_users;
}

// 更新图表数据
function updateCharts(data) {
    // Update product chart
    const productLabels = Object.keys(data.product_stats);
    const productData = Object.values(data.product_stats);
    
    productChart.data.labels = productLabels;
    productChart.data.datasets[0].data = productData;
    productChart.update();

    // Update weekly trend chart
    const weeklyData = [0, 0, 0, 0, 0, 0, 0]; // Initialize array for 7 days
    data.returns.forEach(return_item => {
        const date = new Date(return_item.return_date);
        const dayOfWeek = date.getDay(); // 0 = Sunday, 1 = Monday, etc.
        weeklyData[dayOfWeek]++;
    });
    
    hourlyChart.data.datasets[0].data = weeklyData;
    hourlyChart.update();
}

// 根据日期范围筛选数据
function filterByDate(range) {
    const now = new Date();
    let startDate = new Date();
    let endDate = new Date();
    
    switch(range) {
        case 'today':
            startDate.setHours(0, 0, 0, 0);
            endDate.setHours(23, 59, 59, 999);
            break;
        case 'yesterday':
            startDate.setDate(now.getDate() - 1);
            startDate.setHours(0, 0, 0, 0);
            endDate.setDate(now.getDate() - 1);
            endDate.setHours(23, 59, 59, 999);
            break;
        case 'week':
            // 获取本周一
            const day = now.getDay();
            const diff = now.getDate() - day + (day === 0 ? -6 : 1);
            startDate = new Date(now.setDate(diff));
            startDate.setHours(0, 0, 0, 0);
            // 获取本周日
            endDate = new Date(startDate);
            endDate.setDate(startDate.getDate() + 6);
            endDate.setHours(23, 59, 59, 999);
            break;
        case 'month':
            // 获取本月第一天
            startDate = new Date(now.getFullYear(), now.getMonth(), 1);
            startDate.setHours(0, 0, 0, 0);
            // 获取本月最后一天
            endDate = new Date(now.getFullYear(), now.getMonth() + 1, 0);
            endDate.setHours(23, 59, 59, 999);
            break;
        case 'quarter':
            const currentQuarter = Math.floor(now.getMonth() / 3);
            startDate = new Date(now.getFullYear(), currentQuarter * 3, 1);
            startDate.setHours(0, 0, 0, 0);
            endDate = new Date(now.getFullYear(), (currentQuarter + 1) * 3, 0);
            endDate.setHours(23, 59, 59, 999);
            break;
        case 'year':
            startDate = new Date(now.getFullYear(), 0, 1);
            startDate.setHours(0, 0, 0, 0);
            endDate = new Date(now.getFullYear(), 11, 31);
            endDate.setHours(23, 59, 59, 999);
            break;
    }
    
    fetchData(startDate, endDate);
}

function filterByCustomRange() {
    const startDate = new Date(document.getElementById('startDate').value);
    const endDate = new Date(document.getElementById('endDate').value);
    fetchData(startDate, endDate);
}

function fetchData(startDate, endDate) {
    fetch(`/returns?start_date=${startDate.toISOString()}&end_date=${endDate.toISOString()}`)
        .then(response => response.json())
        .then(data => {
            updateStats(data);
            updateCharts(data);
            updateTable(data.returns);
        })
        .catch(error => console.error('Error:', error));
}

// Date picker modal functions
function showDatePickerModal() {
    const modal = document.getElementById('datePickerModal');
    modal.style.display = 'block';
    
    // Set default dates
    const today = new Date();
    const lastMonth = new Date(today.getFullYear(), today.getMonth() - 1, today.getDate());
    
    document.getElementById('modalStartDate').value = lastMonth.toISOString().split('T')[0];
    document.getElementById('modalEndDate').value = today.toISOString().split('T')[0];
}

function closeDatePickerModal() {
    const modal = document.getElementById('datePickerModal');
    modal.style.display = 'none';
}

function selectPresetRange(preset) {
    const today = new Date();
    let startDate = new Date();
    
    switch(preset) {
        case 'last7days':
            startDate.setDate(today.getDate() - 7);
            break;
        case 'last30days':
            startDate.setDate(today.getDate() - 30);
            break;
        case 'last90days':
            startDate.setDate(today.getDate() - 90);
            break;
        case 'lastYear':
            startDate.setFullYear(today.getFullYear() - 1);
            break;
    }
    
    document.getElementById('modalStartDate').value = startDate.toISOString().split('T')[0];
    document.getElementById('modalEndDate').value = today.toISOString().split('T')[0];
}

function applyDateRange() {
    const startDate = new Date(document.getElementById('modalStartDate').value);
    const endDate = new Date(document.getElementById('modalEndDate').value);
    
    // Set time to start and end of day
    startDate.setHours(0, 0, 0, 0);
    endDate.setHours(23, 59, 59, 999);
    
    fetchData(startDate, endDate);
    closeDatePickerModal();
}

// Update table with data
function updateTable(returns) {
    const tbody = document.querySelector('.returns-table tbody');
    const noRecordsMessage = document.getElementById('noRecordsMessage');
    
    // Clear existing rows except the no records message
    const rows = tbody.querySelectorAll('tr:not(#noRecordsMessage)');
    rows.forEach(row => row.remove());
    
    if (!returns || returns.length === 0) {
        noRecordsMessage.style.display = 'table-row';
        return;
    }
    
    noRecordsMessage.style.display = 'none';
    
    returns.forEach(return_item => {
        const row = document.createElement('tr');
        row.innerHTML = `
            <td>${return_item.id}</td>
            <td>${return_item.customer_id}</td>
            <td>${return_item.product_id}</td>
            <td>${return_item.reward_value}</td>
            <td>${return_item.return_date}</td>
            <td>
                <button class="action-btn view-btn" onclick="toggleDetails(this)">View</button>
                <button class="action-btn details-btn" onclick="showDetailsModal(this)" style="display: none;">Details</button>
            </td>
        `;
        tbody.appendChild(row);
        
        const detailsRow = document.createElement('tr');
        detailsRow.className = 'details-row';
        detailsRow.style.display = 'none';
        detailsRow.innerHTML = `
            <td colspan="6">
                <div class="details-content">
                    <p><strong>ID:</strong> ${return_item.id}</p>
                    <p><strong>Customer ID:</strong> ${return_item.customer_id}</p>
                    <p><strong>Product ID:</strong> ${return_item.product_id}</p>
                    <p><strong>Reward Value:</strong> ${return_item.reward_value}</p>
                    <p><strong>Return Date:</strong> ${return_item.return_date}</p>
                </div>
            </td>
        `;
        tbody.appendChild(detailsRow);
    });
}

// 刷新数据
function refreshData() {
    // 根据当前的筛选条件重新获取数据
    switch(currentFilter) {
        case 'today':
            filterByDate('today');
            break;
        case 'week':
            filterByDate('week');
            break;
        case 'month':
            filterByDate('month');
            break;
        case 'custom':
            const customDate = document.getElementById('customDate').value;
            if (customDate) {
                filterByDate(customDate);
            }
            break;
    }
}

// Table interaction functions
function toggleDetails(button) {
    const row = button.closest('tr');
    const detailsRow = row.nextElementSibling;
    const detailsBtn = row.querySelector('.details-btn');
    
    if (detailsRow.style.display === 'none') {
        detailsRow.style.display = 'table-row';
        detailsBtn.style.display = 'inline-block';
        button.textContent = 'Hide';
    } else {
        detailsRow.style.display = 'none';
        detailsBtn.style.display = 'none';
        button.textContent = 'View';
    }
}

// Modal functions
const modal = document.getElementById('detailsModal');
const closeBtn = document.querySelector('.close');

function showDetailsModal(button) {
    const row = button.closest('tr');
    const detailsContent = row.nextElementSibling.querySelector('.details-content').innerHTML;
    document.getElementById('modalContent').innerHTML = detailsContent;
    modal.style.display = 'block';
}

// Close modal when clicking the close button
closeBtn.onclick = function() {
    modal.style.display = 'none';
}

// Close modal when clicking outside
window.onclick = function(event) {
    if (event.target == modal) {
        modal.style.display = 'none';
    }
}

// Add double-click handler for rows
document.querySelectorAll('.returns-table tbody tr').forEach(row => {
    row.addEventListener('dblclick', function() {
        const detailsContent = this.nextElementSibling.querySelector('.details-content').innerHTML;
        document.getElementById('modalContent').innerHTML = detailsContent;
        modal.style.display = 'block';
    });
});
